@@ -10,13 +10,12 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from unittest import mock
 
 import ddt
 from keystoneauth1 import exceptions as kaexception
-import mock
 import requests
 
-
 from ironic.common import context
 from ironic.common import keystone
 from ironic.common import nova
@@ -69,23 +68,23 @@ class NovaApiTestCase(base.TestCase):
               [{'events': [{'code': 400}]}, 207, False],
               # This (response 207, event code 200) will never happen IRL
               [{'events': [{'code': 200}]}, 207, True])
-    @mock.patch.object(nova, '_get_nova_adapter')
+    @mock.patch.object(nova, '_get_nova_adapter', autospec=True)
     def test_power_update(self, nova_result, resp_status, exp_ret,
                           mock_adapter, mock_log):
         server_ids = ['server-id-1', 'server-id-2']
         nova_adapter = mock.Mock()
-        with mock.patch.object(nova_adapter, 'post') as mock_post_event:
-            post_resp_mock = requests.Response()
+        mock_post_event = nova_adapter.post
+        post_resp_mock = requests.Response()
 
-            def json_func():
-                return nova_result
-            post_resp_mock.json = json_func
-            post_resp_mock.status_code = resp_status
-            mock_adapter.return_value = nova_adapter
-            mock_post_event.return_value = post_resp_mock
-            for server in server_ids:
-                result = self.api.power_update(self.ctx, server, 'power on')
-                self.assertEqual(exp_ret, result)
+        def json_func():
+            return nova_result
+        post_resp_mock.json = json_func
+        post_resp_mock.status_code = resp_status
+        mock_adapter.return_value = nova_adapter
+        mock_post_event.return_value = post_resp_mock
+        for server in server_ids:
+            result = self.api.power_update(self.ctx, server, 'power on')
+            self.assertEqual(exp_ret, result)
 
         mock_adapter.assert_has_calls([mock.call(), mock.call()])
         req_url = '/os-server-external-events'
@@ -113,14 +112,14 @@ class NovaApiTestCase(base.TestCase):
             expected = ("Nova event response: %s.", nova_result['events'][0])
             mock_log.debug.assert_called_with(*expected)
 
-    @mock.patch.object(nova, '_get_nova_adapter')
+    @mock.patch.object(nova, '_get_nova_adapter', autospec=True)
     def test_invalid_power_update(self, mock_adapter, mock_log):
         nova_adapter = mock.Mock()
-        with mock.patch.object(nova_adapter, 'post') as mock_post_event:
-            result = self.api.power_update(self.ctx, 'server', None)
-            self.assertFalse(result)
-            expected = ('Invalid Power State %s.', None)
-            mock_log.error.assert_called_once_with(*expected)
+        mock_post_event = nova_adapter.post
+        result = self.api.power_update(self.ctx, 'server', None)
+        self.assertFalse(result)
+        expected = ('Invalid Power State %s.', None)
+        mock_log.error.assert_called_once_with(*expected)
 
         mock_adapter.assert_not_called()
         mock_post_event.assert_not_called()
@@ -131,7 +130,8 @@ class NovaApiTestCase(base.TestCase):
                   'server_uuid': 'server-id-1',
                   'tag': 'POWER_OFF'}]
         nova_result = requests.Response()
-        with mock.patch.object(nova_adapter, 'post') as mock_post_event:
+        with mock.patch.object(nova_adapter, 'post',
+                               autospec=True) as mock_post_event:
             for stat_code in (500, 404, 400):
                 mock_log.reset_mock()
                 nova_result.status_code = stat_code
@@ -156,22 +156,22 @@ class NovaApiTestCase(base.TestCase):
               {'events': []},
               {'events': None},
               {})
-    @mock.patch.object(nova, '_get_nova_adapter')
+    @mock.patch.object(nova, '_get_nova_adapter', autospec=True)
     def test_power_update_invalid_reponse_format(self, nova_result,
                                                  mock_adapter, mock_log):
         nova_adapter = mock.Mock()
-        with mock.patch.object(nova_adapter, 'post') as mock_post_event:
-            post_resp_mock = requests.Response()
+        mock_post_event = nova_adapter.post
+        post_resp_mock = requests.Response()
 
-            def json_func():
-                return nova_result
+        def json_func():
+            return nova_result
 
-            post_resp_mock.json = json_func
-            post_resp_mock.status_code = 207
-            mock_adapter.return_value = nova_adapter
-            mock_post_event.return_value = post_resp_mock
-            result = self.api.power_update(self.ctx, 'server-id-1', 'power on')
-            self.assertFalse(result)
+        post_resp_mock.json = json_func
+        post_resp_mock.status_code = 207
+        mock_adapter.return_value = nova_adapter
+        mock_post_event.return_value = post_resp_mock
+        result = self.api.power_update(self.ctx, 'server-id-1', 'power on')
+        self.assertFalse(result)
 
         mock_adapter.assert_has_calls([mock.call()])
         req_url = '/os-server-external-events'
@@ -193,7 +193,7 @@ class NovaApiTestCase(base.TestCase):
         self.assertFalse(result)
         mock_adapter.assert_not_called()
 
-    @mock.patch.object(nova, '_get_nova_adapter')
+    @mock.patch.object(nova, '_get_nova_adapter', autospec=True)
     def test_power_update_failed_no_nova_auth_url(self, mock_adapter,
                                                   mock_log):
         server = 'server-id-1'
